/**
 * This source file is part of OgreColladaPlugin
 * an addon for OGRE (Object-oriented Graphics Rendering Engine)
 * For the latest info, see http://www.ogre3d.org/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.

 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 * http://www.gnu.org/copyleft/lesser.txt.
 *
 * @author	Philipp Hartl, Jeff Doyle (nfz)
 * @see		README
 */

#ifndef __COLLADA_ASSET_H__
#define __COLLADA_ASSET_H__

#include "OgreColladaPrerequisites.h"

namespace Ogre
{
    /**  Collada assets define asset management information regarding its parent element.
    */
    class ColladaAsset
    {
    public:
        ColladaAsset(xmlNode *node) : mNode(node), mUpAxis(Y_UP) {}
        ~ColladaAsset();

		/**
		 * import <asset> node
		 *
         */
		bool doImport();
        /** Get the up axis descriptive information for the coordinate system of the geometric data.
         * @return the axis the points up: X_UP, Y_UP, Z_UP
         */
        UpAxis getUpAxis() const { return mUpAxis; }

    protected:
		xmlNode			*mNode;		// the node structure to import
        UpAxis          mUpAxis;

    };

}

#endif // __COLLADA_ASSET_H__
