/**
 * This source file is part of OgreColladaPlugin
 * an addon for OGRE (Object-oriented Graphics Rendering Engine)
 * For the latest info, see http://www.ogre3d.org/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.

 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 * http://www.gnu.org/copyleft/lesser.txt.
 *
 * @author	Philipp Hartl
 * @see		README
 */

#ifndef __COLLADA_MATERIAL_H__
#define __COLLADA_MATERIAL_H__

#include "OgreColladaPrerequisites.h"
#include "OgreColladaEntity.h"

#include "OgreColourValue.h"
#include "OgreCommon.h"

namespace Ogre
{
	struct ColladaMaterialPass;

	typedef std::vector<ColladaMaterialPass *> ColladaMaterialPassPtrVector;
	typedef std::vector<ColladaTexture *> ColladaTexturePtrVector;

	/**
	 * a material object
	 */
	class ColladaMaterial : public ColladaEntity
	{
	public:
		ColladaMaterial(ColladaDocument *doc, xmlNode *n);
		virtual ~ColladaMaterial(void);

		/**
		 * create and load a ogre material
		 * with multiple passes
		 */
		virtual MovableObject *getOgreInstance(void) const;

		/**
		 * import the <material> node
		 *
		 * @return	true if succeeds
		 */
		virtual bool doImport(void);

		virtual EntityTypes getEntityType(void) const { return MATERIAL; }

	private:
		ColladaMaterialPassPtrVector mPasses;	// all material passes
		
		/**
		 * import rendering pass properties from <pass> node
		 * 
		 * @param	node	the <pass> node
		 * @return	a pointer on a material pass structure
		 */
		ColladaMaterialPass *importPass(xmlNode *pass);
	};

	/**
	 * all material properties for one rendering pass
	 * @see	collada 1.3.1 specification for details
	 */
	struct ColladaMaterialPass
	{
		ShadeOptions mMode;
		ColourValue	mAmbient, mDiffuse, mSpecular, mEmission, mTransparent, mReflective;
		float		mShininess, mTransparency, mReflectivity;		
		ColladaTexturePtrVector	mTextures;		// a texture vector, for multitexturing purpose
	};
}

#endif // __COLLADA_MATERIAL_H__
