/**
 * This source file is part of OgreColladaPlugin
 * an addon for OGRE (Object-oriented Graphics Rendering Engine)
 * For the latest info, see http://www.ogre3d.org/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.

 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 * http://www.gnu.org/copyleft/lesser.txt.
 *
 * @author	Philipp Hartl
 * @see		README
 */

#ifndef __COLLADA_PREREQUISITES_H__
#define __COLLADA_PREREQUISITES_H__

// libxml stuff
// #define LIBXML_STATIC
#include <libxml/parser.h>
#include <libxml/tree.h>

// Ogre stuff
#include "OgreLogManager.h"
// std includes
//#include <vector>

namespace Ogre
{
    #define COLLADA_VERSION_MAJOR 1
    #define COLLADA_VERSION_MINOR 3
    #define COLLADA_VERSION_PATCH 1

	/**
	 * typedefs
	 */
	typedef std::vector<xmlNode *> xmlNodePtrVector;
	typedef std::vector<int> intVector;
	typedef std::vector<float> floatVector;

    enum UpAxis
    {
        X_UP = 0,
        Y_UP,
        Z_UP
    };

	/**
	 * pre-declared structures
	 */
	
	class ColladaEntity;
	template <class T> class ColladaLibrary;

	class ColladaBoundingBox;
	class ColladaCamera;
	class ColladaDocument;
	class ColladaGeometry;
	class ColladaImage;
	class ColladaLibraryContainer;
	class ColladaLight;
	class ColladaMaterial;
	class ColladaTexture;
	class ColladaSceneNode;
    class ColladaAsset;
}

#if (OGRE_PLATFORM == OGRE_PLATFORM_WIN32) && !defined(__MINGW32__)
#   ifdef PLUGIN_COLLADA_EXPORT 
#       define _OgreColladaExport __declspec(dllexport) 
#   else 
#       define _OgreColladaExport __declspec(dllimport) 
#   endif 
#else 
#   define _OgreColladaExport 
#endif 

#endif // __COLLADA_PREREQUISITES_H__
