/**
 * This source file is part of OgreColladaPlugin
 * an addon for OGRE (Object-oriented Graphics Rendering Engine)
 * For the latest info, see http://www.ogre3d.org/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.

 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 * http://www.gnu.org/copyleft/lesser.txt.
 *
 * @author	Philipp Hartl
 * @see		README
 */

#ifndef __COLLADA_SYNTAX_H__
#define __COLLADA_SYNTAX_H__

namespace Ogre
{

/**
 * collada syntax "CS_"
 * Element (tag) names start with "ELM_"
 * attribute (property) names starts with "ATR_{ELement name}"
 * possible attribute value names starts with "VAL_"
 * 
 * @see		collada specification
 * @version	1.3.1
 */

// Elements
#define CS_ELM_COLLADA		"COLLADA"
#define CS_ELM_ACCESSOR		"accessor"
#define CS_ELM_ASSET		"asset"
#define CS_ELM_BOUNDINGBOX	"boundingbox"
#define CS_ELM_EXTRA		"extra"
#define CS_ELM_FLOAT_ARRAY	"float_array"
#define CS_ELM_INPUT		"input"
#define CS_ELM_INSTANCE		"instance"
#define CS_ELM_LIBRARY		"library"
#define CS_ELM_LOOKAT		"lookat"
#define CS_ELM_MATRIX		"matrix"
#define CS_ELM_MAX			"max"
#define CS_ELM_MESH			"mesh"
#define CS_ELM_MIN			"min"
#define CS_ELM_NODE			"node"
#define CS_ELM_OPTICS		"optics"
#define CS_ELM_IMAGER		"imager"
#define CS_ELM_P			"p"
#define CS_ELM_PASS			"pass"
#define CS_ELM_PARAM		"param"
#define CS_ELM_PERSPECTIVE	"perspective"
#define CS_ELM_POLYGONS		"polygons"
#define CS_ELM_PROGRAM		"program"
#define CS_ELM_ROTATE		"rotate"
#define CS_ELM_SCALE		"scale"
#define CS_ELM_SCENE		"scene"
#define CS_ELM_SHADER		"shader"
#define CS_ELM_SKEW			"skew"
#define CS_ELM_SOURCE		"source"
#define CS_ELM_TECHNIQUE	"technique"
#define CS_ELM_TRIANGLES	"triangles"
#define CS_ELM_TRANSLATE	"translate"
#define CS_ELM_UPAXIS       "up_axis"
#define CS_ELM_VERTICES		"vertices"

// common attributes
#define CS_ATR_COUNT		"count"
#define CS_ATR_ID			"id"
#define CS_ATR_IDX			"idx"
#define CS_ATR_NAME			"name"
#define CS_ATR_PROFILE		"profile"
#define CS_ATR_TYPE			"type"
#define CS_ATR_URL			"url"

// specific Attributes
#define CS_ATR_INPUT_SEMANTIC	"semantic"
#define CS_ATR_INPUT_SOURCE		"source"
#define CS_ATR_INPUT_MATERIAL	"material"

#define CS_ATR_ACCESSOR_STRIDE	"stride"

#define CS_ATR_IMAGE_DEPTH		"depth"
#define CS_ATR_IMAGE_FORMAT		"format"
#define CS_ATR_IMAGE_HEIGHT		"height"
#define CS_ATR_IMAGE_SOURCE		"source"
#define CS_ATR_IMAGE_WIDTH		"width"

// possible values of attributes
#define CS_VAL_CAMERA_TYPE_ORTHOGRAPHIC	"ORTHOGRAPHIC"
#define CS_VAL_CAMERA_TYPE_PERSPECTIVE	"PERSPECTIVE"

#define CS_VAL_CAMERA_PARAM_BOTTOM		"BOTTOM"
#define CS_VAL_CAMERA_PARAM_LEFT		"LEFT"
#define CS_VAL_CAMERA_PARAM_RIGHT		"RIGHT"
#define CS_VAL_CAMERA_PARAM_TOP			"TOP"
#define CS_VAL_CAMERA_PARAM_XFOV		"XFOV"
#define CS_VAL_CAMERA_PARAM_YFOV		"YFOV"
#define CS_VAL_CAMERA_PARAM_ZFAR		"ZFAR"
#define CS_VAL_CAMERA_PARAM_ZNEAR		"ZNEAR"

#define CS_VAL_INPUT_SEMANTIC_COLOR		"COLOR"
#define CS_VAL_INPUT_SEMANTIC_IMAGE		"IMAGE"
#define CS_VAL_INPUT_SEMANTIC_NORMAL	"NORMAL"
#define CS_VAL_INPUT_SEMANTIC_POSITION	"POSITION"
#define CS_VAL_INPUT_SEMANTIC_TEXCOORD	"TEXCOORD"
#define CS_VAL_INPUT_SEMANTIC_TEXTURE	"TEXTURE"
#define CS_VAL_INPUT_SEMANTIC_VERTEX	"VERTEX"

#define CS_VAL_LIBRARY_TYPE_ANIMATION	"ANIMATION"
#define CS_VAL_LIBRARY_TYPE_CAMERA		"CAMERA"
#define CS_VAL_LIBRARY_TYPE_CODE		"CODE"
#define CS_VAL_LIBRARY_TYPE_CONTROLLER	"CONTROLLER"
#define CS_VAL_LIBRARY_TYPE_GEOMETRY	"GEOMETRY"
#define CS_VAL_LIBRARY_TYPE_IMAGE		"IMAGE"
#define CS_VAL_LIBRARY_TYPE_LIGHT		"LIGHT"
#define CS_VAL_LIBRARY_TYPE_MATERIAL	"MATERIAL"
#define CS_VAL_LIBRARY_TYPE_PROGRAM		"PROGRAM"
#define CS_VAL_LIBRARY_TYPE_TEXTURE		"TEXTURE"

#define CS_VAL_LIGHT_PARAM_ANGLE				"ANGLE"
#define CS_VAL_LIGHT_PARAM_ATTENUATION			"ATTENUATION"
#define CS_VAL_LIGHT_PARAM_ATTENUATION_SCALE	"ATTENUATION_SCALE"
#define CS_VAL_LIGHT_PARAM_COLOR				"COLOR"
#define CS_VAL_LIGHT_PARAM_FALLOFF				"FALLOFF"
#define CS_VAL_LIGHT_PARAM_FALLOFF_SCALE		"FALLOFF_SCALE"

#define CS_VAL_LIGHT_FUNCTION_CONSTANT		"CONSTANT"
#define CS_VAL_LIGHT_FUNCTION_LINEAR		"LINEAR"
#define CS_VAL_LIGHT_FUNCTION_QUADRATIC		"QUADRATIC"

#define CS_VAL_LIGHT_TYPE_AMBIENT			"AMBIENT"
#define CS_VAL_LIGHT_TYPE_DIRECTIONAL		"DIRECTIONAL"
#define CS_VAL_LIGHT_TYPE_POINT				"POINT"
#define CS_VAL_LIGHT_TYPE_SPOT				"SPOT"

#define CS_VAL_MATERIAL_PARAM_AMBIENT		"AMBIENT"
#define CS_VAL_MATERIAL_PARAM_DIFFUSE		"DIFFUSE"
#define CS_VAL_MATERIAL_PARAM_EMISSION		"EMISSION"
#define CS_VAL_MATERIAL_PARAM_REFLECTIVE	"REFLECTIVE"
#define CS_VAL_MATERIAL_PARAM_REFLECTIVITY	"REFLECTIVITY"
#define CS_VAL_MATERIAL_PARAM_SHININESS		"SHININESS"
#define CS_VAL_MATERIAL_PARAM_SPECULAR		"SPECULAR"
#define CS_VAL_MATERIAL_PARAM_TRANSPARENCY	"TRANSPARENCY"
#define CS_VAL_MATERIAL_PARAM_TRANSPARENT	"TRANSPARENT"

#define CS_VAL_NODE_TYPE_JOINT				"JOINT"
#define CS_VAL_NODE_TYPE_NODE				"NODE"

#define CS_VAL_PROGRAM_URL_CONSTANT			"CONSTANT"
#define CS_VAL_PROGRAM_URL_LAMBERT			"LAMBERT"
#define CS_VAL_PROGRAM_URL_PHONG			"PHONG"

#define CS_VAL_TEXTURE_PARAM_AMBIENT		"AMBIENT"
#define CS_VAL_TEXTURE_PARAM_DIFFUSE		"DIFFUSE"
#define CS_VAL_TEXTURE_PARAM_EMISSION		"EMISSION"
#define CS_VAL_TEXTURE_PARAM_SHININESS		"SHININESS"
#define CS_VAL_TEXTURE_PARAM_SPECULAR		"SPECULAR"

#define CS_VAL_TECHNIQUE_PROFILE_COMMON		"COMMON"

// possible values of content
#define CS_VAL_ASSET_UPAXIS_X "X_UP"
#define CS_VAL_ASSET_UPAXIS_Y "Y_UP"
#define CS_VAL_ASSET_UPAXIS_Z "Z_UP"


}

#endif // __COLLADA_SYNTAX_H__
